;;;;
;;;; gaf-v1 -- gschem specification used by gaf-read and gaf-write.
;;;;
;;;; Copyright 2007 Dean Ferreyra, All rights reserved
;;;;
;;;; This file is part of octw-guile-geda.
;;;;
;;;; octw-guile-geda is free software: you can redistribute it and/or modify
;;;; it under the terms of the GNU General Public License as published by
;;;; the Free Software Foundation, either version 3 of the License, or
;;;; (at your option) any later version.
;;;; 
;;;; octw-guile-geda is distributed in the hope that it will be useful,
;;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;;; GNU General Public License for more details.
;;;; 
;;;; You should have received a copy of the GNU General Public License
;;;; along with octw-guile-geda.  If not, see <http://www.gnu.org/licenses/>.
;;;;
;;;; $Id: gaf-v1.scm,v 1.1 2008-09-04 03:40:47 dean Exp $

(define-module (octw gaf-v1))

(use-modules (srfi srfi-1)
	     (srfi srfi-13)
	     (srfi srfi-19))
(use-modules (octw gaf))
(use-syntax (ice-9 syncase))

(export with-gaf-v1-specs
        gaf-gen-v1-specs
        text-lines
        text-lines-set!
        gaf-composite-attribute-text-obj
        gaf-composite-attribute-value
        gaf-composite-attribute-value-set!
        gaf-composite-attribute-add!
        )

(define-syntax with-gaf-v1-specs
  (lambda (form)
    (syntax-case form ()
      ((_ macro args ...)
       (syntax
        (macro ((version
                 #\v
                 ((version int)
                  (fileformat_version int)))
                (line
                 #\L
                 ((x1 int)
                  (y1 int)
                  (x2 int)
                  (y2 int)
                  (color int)
                  (width int)
                  (capstyle int)
                  (dashstyle int)
                  (dashlength int)
                  (dashspace int)))
                ;; ;; Picture not handled yet...
                ;; (picture
                ;;  #\G
                ;;  ((x int)
                ;;   (y int)
                ;;   (width int)
                ;;   (height int)
                ;;   (angle int)
                ;;   (mirrored char)
                ;;   (embedded char)
                ;;   (filename string)
                ;;   ;; encoded-picture-data and encoded-picture-end appear only if
                ;;   ;; data is embedded, in which case you can get multiple
                ;;   ;; encoded-picture-data entries
                ;;   (encoded-picture-data string)
                ;;   (encoded-picture-end string)))
                ;;
                (box
                 #\B
                 ((x int)
                  (y int)
                  (width int)
                  (height int)
                  (color int)
                  (width int)
                  (capstyle int)
                  (dashstyle int)
                  (dashlength int)
                  (dashspace int)
                  (filltype int)
                  (fillwidth int)
                  (angle1 int)
                  (pitch1 int)
                  (angle2 int)
                  (pitch2 int)))
                (circle
                 #\V
                 ((x int)
                  (y int)
                  (radius int)
                  (color int)
                  (width int)
                  (capstyle int)
                  (dashstyle int)
                  (dashlength int)
                  (dashspace int)
                  (filltype int)
                  (fillwidth int)
                  (angle1 int)
                  (pitch1 int)
                  (angle2 int)
                  (pitch2 int)))
                (arc
                 #\A
                 ((x int)
                  (y int)
                  (radius int)
                  (startangle int)
                  (sweepangle int)
                  (color int)
                  (width int)
                  (capstyle int)
                  (dashstyle int)
                  (dashlength int)
                  (dashspace int)))
                (text
                 #\T
                 ((x int)
                  (y int)
                  (color int)
                  (size int)
                  (visibility int)
                  (show_name_value int)
                  (angle int)
                  (alignment int)
                  (num_lines int)))
                (net
                 #\N
                 ((x1 int)
                  (y1 int)
                  (x2 int)
                  (y2 int)
                  (color int)))
                (bus
                 #\U
                 ((x1 int)
                  (y1 int)
                  (x2 int)
                  (y2 int)
                  (color int)
                  (ripperdir int)))
                (pin
                 #\P
                 ((x1 int)
                  (y1 int)
                  (x2 int)
                  (y2 int)
                  (color int)
                  (pintype int)
                  (whichend int)))
                (component
                 #\C
                 ((x int)
                  (y int)
                  (selectable int)
                  (angle int)
                  (mirror int)
                  (basename string)))
                (font
                 #\F
                 ((character char)
                  (width int)
                  (flag int)))
                (embedded-start
                 #\[
                 ())
                (embedded-end
                 #\]
                 ())
                (attributes-start
                 #\{
                 ())
                (attributes-end
                 #\}
                 ()))
          args ...))))))

(define-syntax gaf-gen-v1-specs
  (lambda (form)
    (syntax-case form ()
      ((_)
       (syntax
        (gaf-gen with-gaf-v1-specs))))))

(define (text-lines obj)
  (list-ref obj 10))

(define (text-lines-set! obj lines)
  (set-car! (drop obj 10) lines)
  obj)

(define (gaf-composite-attribute-text-obj obj key)
  (gaf-base-composite-attribute-text-obj text-lines obj key))

(define (gaf-composite-attribute-value obj key)
  (gaf-base-composite-attribute-value text-lines obj key))

(define (gaf-composite-attribute-value-set! obj key val)
  (gaf-base-composite-attribute-value-set! text-lines obj key val))

(define (gaf-composite-attribute-add! obj key val text-obj-template)
  (gaf-base-composite-attribute-add! text-lines-set! obj key val text-obj-template))
