;;;;
;;;; pads-scripting -- Functions and macros that help with writing
;;;; scripts using the (octw pads) module.
;;;;
;;;; Copyright 2007 Dean Ferreyra, All rights reserved
;;;;
;;;; This file is part of octw-guile-geda.
;;;;
;;;; octw-guile-geda is free software: you can redistribute it and/or modify
;;;; it under the terms of the GNU General Public License as published by
;;;; the Free Software Foundation, either version 3 of the License, or
;;;; (at your option) any later version.
;;;; 
;;;; octw-guile-geda is distributed in the hope that it will be useful,
;;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;;; GNU General Public License for more details.
;;;; 
;;;; You should have received a copy of the GNU General Public License
;;;; along with octw-guile-geda.  If not, see <http://www.gnu.org/licenses/>.
;;;;
;;;; $Id: pads-scripting.scm,v 1.1 2008-09-04 03:40:47 dean Exp $

(define-module (octw pads-scripting))

(export distance->cmil
        string->cmil
        evaled-string->cmil
        distance-units-option-spec
        set-distance-option!
        evaled-option
        set-evaled-option!
        percent)

(use-syntax (ice-9 syncase))
(use-modules (ice-9 getopt-long)
             (ice-9 optargs)
             (octw anaphoric)
             (octw pads))

;;; distance->cmil
;;; string->cmil
;;; evaled-string->cmil
;;;
;;; Converts various forms to cmils using getopt-long options.  Expects
;;; option 'mm for millimeters, 'cmil for centimils, or neither for plain mils.

(define (distance->cmil options dist)
  (let ((scale (cond ((option-ref options 'mm #f)
                      mm->centimils)
                     ((option-ref options 'cmil #f)
                      identity)
                     ((option-ref options 'in #f)
                      in->centimils)
                     (else
                      mils->centimils))))
    (scale dist)))

(define (string->cmil options s)
  (distance->cmil options (string->number s)))

(define (evaled-string->cmil options s)
  (distance->cmil options (eval-string s)))

;;; Prepackaged option specification fragment for handling units.
(define (distance-units-option-spec)
  '((mm (single-char #\m))
    (cmil (single-char #\c))
    (in (single-char #\i))))

;;; If OPTION-SYM option appears in OPTIONS, evaluate the string,
;;; convert into centimils, and assign VAR to the result.
(define-syntax set-distance-option!
  (lambda (form)
    (syntax-case form ()
      ((_ var options option-sym)
       (syntax
        (let ((options-e options))
          (let ((str (option-ref options-e option-sym #f)))
            (if str
                (set! var (evaled-string->cmil options-e str))))))))))

;;; If OPTION-SYM option appears in OPTIONS, evaluate the string and
;;; assign VAR to the result.  Optionally apply FN to the result
;;; before assigning to VAR.
(define-syntax set-evaled-option!
  (lambda (form)
    (syntax-case form ()
      ((_ var options option-sym)
       (syntax
        (set-evaled-option! var options option-sym identity)))
      ((_ var options option-sym fn)
       (syntax
        ;; XXX Can't use awhen here for some reason
        (let ((str (option-ref options option-sym #f)))
          (when str
            (set! var (fn (eval-string str))))))))))

(define* (evaled-option options option-sym #:optional default)
  (aif (option-ref options option-sym #f)
       (eval-string it)
       default))
  
(define (percent n)
  (/ n 100))
